@@ -22,13 +22,13 @@
 #include "exec.h"
 #include "globals.h"
 #include "config.h"
+#include "xclip.h"
 
 /* "local global" */
 int cur_colorscheme_nr = -1;
 int cur_filterscheme_nr = -1;
 int cur_editscheme_nr = -1;
 
-
 mybool_t config_yes_no(char *what)
 {
 	if (what[0] == '1' || strcasecmp(what, "yes") == 0 || strcasecmp(what, "y") == 0 || strcasecmp(what, "on") == 0)
@@ -729,6 +729,8 @@ void set_check_mail(int linenr, char *cmd, char *par)
 void set_tab_stop(int linenr, char *cmd, char *par)
 {
 	tab_width = atoi(par);
+	if (tab_width == 0)
+		config_error_exit(linenr, "tab_stop: value cannot be 0.\n");
 }
 
 void set_tail(int linenr, char *cmd, char *par)
@@ -830,11 +832,24 @@ void set_scrollback_show_winnrs(int linenr, char *cmd, char *par)
 	default_sb_showwinnr = config_yes_no(par);
 }
 
-void  set_wordwrapmaxlength(int linenr, char *cmd, char *par)
+void set_wordwrapmaxlength(int linenr, char *cmd, char *par)
 {
 	wordwrapmaxlength = get_value_arg("wordwrapmaxlength", par, VAL_POSITIVE);
 }
 
+void set_xclip(int linenr, char *cmd, char *par)
+{
+	if (file_exist(par) == -1)
+		error_exit(TRUE, FALSE, "xclip binary '%s' does not exist");
+
+	xclip = strdup(par);
+}
+
+void set_map_delete_as_backspace(int linenr, char *cmd, char *par)
+{
+	map_delete_as_backspace = config_yes_no(par);	
+}
+
 void set_searchhistory_file(int linenr, char *cmd, char *par)
 {
 	if (par[0] == 0x00)
@@ -1058,6 +1073,7 @@ config_file_keyword cf_entries[] = {
 	{ "include", do_load_config },
 	{ "inverse", set_inverse_attrs },
 	{ "line_ts_format", set_line_ts_format },
+	{ "map_delete_as_backspace", set_map_delete_as_backspace },
 	{ "markerline_char", set_markerline_char },
 	{ "markerline_color", set_markerline_attrs },
 	{ "markerline_timestamp", set_markerline_timestamp },
@@ -1103,7 +1119,8 @@ config_file_keyword cf_entries[] = {
 	{ "usefilterscheme", use_filterscheme },
 	{ "warn_closed", set_warn_closed },
 	{ "window_number", set_window_number },
-	{ "wordwrapmaxlength", set_wordwrapmaxlength }
+	{ "wordwrapmaxlength", set_wordwrapmaxlength },
+	{ "xclip", set_xclip }
 };
 
 int find_config_entry_in_dispatch_table(char *cmd_name)
@@ -1226,10 +1243,11 @@ void load_configfile_wrapper(char *config_file)
 
 		if (home)
 			snprintf(path, path_max, "%s/.multitailrc", home);
-		else
+		else if (pp)
 			snprintf(path, path_max, "%s/.multitailrc", pp -> pw_dir);
 
-		do_load_config(-1, NULL, path);
+		if (pp || home)
+			do_load_config(-1, NULL, path);
 
 		myfree(path);
 	}
