@@ -3,6 +3,7 @@
 #include <regex.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -600,6 +601,7 @@ char *help_scrollback_help[] = {
 		"lines. This size can be set with the ^-m^ command-",
 		"line parameter or the ^m^-key in the main menu.",
 		" ^c^     set colors",
+		" ^x^     copy contents to X clipboard (xclip)",
 		" ^f^/^/^   search for a string in the buffer",
 		" ^n^     find the next occurence",
 		" ^Y^     toggle linewrap. if linewrap is disabled,",
@@ -610,6 +612,7 @@ char *help_scrollback_help[] = {
 		" ^t^     show a list of window-nrs and what they",
 		"       contain. can also display statistics.",
 		" ^s^     save the buffer to a file",
+		" ^F9^/^ctrl+w^ switch to full screen mode (and back)",
 		"Press ^ctrl+g^ to abort.",
 		NULL
 	};
@@ -1235,6 +1238,9 @@ char ansi_terminal(void)
 {
 	const char *term = getenv("TERM");
 
+	if (!isatty(1) || !isatty(2))
+		return 0;
+
 	if (!term)
 		return 0;
 
