@@ -447,11 +447,9 @@ int draw_tab(NEWWIN *win)
 
 		for(loop=0; loop<move; loop++)
 			waddch(win -> win, ' ');
-
-		return move;
 	}
 
-	return 0;
+	return 1;
 }
 
 
@@ -1390,13 +1388,12 @@ void update_statusline(NEWWIN *status, int win_nr, proginfo *cur)
 		else if (mail)
 			wprintw(status -> win, " You've got mail!");
 
-		dx = getcurx(status -> win);
-		if (dx >= (statusline_len + 13))
+		if (win_width >= (statusline_len + 13))
 		{
 			if (cur -> paused)
 			{
 				color_on(status, find_colorpair(COLOR_YELLOW, -1, 0));
-				mvwprintw(status -> win, 0, dx - 10, "  Paused  ");
+				mvwprintw(status -> win, 0, win_width - 10, "  Paused  ");
 				color_off(status, find_colorpair(COLOR_YELLOW, -1, 0));
 			}
 			else if (cur -> wt == WT_COMMAND)
@@ -1405,9 +1402,9 @@ void update_statusline(NEWWIN *status, int win_nr, proginfo *cur)
 
 				total_info_len = statusline_len + 12;
 
-				if (vmsize != -1 && dx >= (statusline_len + 30))
+				if (vmsize != -1 && win_width >= (statusline_len + 30))
 				{
-					int str_x = dx - strlen(timestamp) - 30;
+					int str_x = win_width - strlen(timestamp) - 30;
 					char *vmsize_str = amount_to_str(vmsize);
 					mvwprintw(status -> win, 0, str_x, "%6s (VMsize) %5d (PID) - %s", vmsize_str, cur -> pid, timestamp);
 					myfree(vmsize_str);
@@ -1418,19 +1415,19 @@ void update_statusline(NEWWIN *status, int win_nr, proginfo *cur)
 				{
 					if (cur -> last_exit_rc != 0)
 					{
-						mvwprintw(status -> win, 0, dx - strlen(timestamp) - 26, "Last rc: %d, %5d (PID) - %s", WEXITSTATUS(cur -> last_exit_rc), cur -> pid, timestamp);
+						mvwprintw(status -> win, 0, win_width - strlen(timestamp) - 26, "Last rc: %d, %5d (PID) - %s", WEXITSTATUS(cur -> last_exit_rc), cur -> pid, timestamp);
 					}
 					else
-						mvwprintw(status -> win, 0, dx - strlen(timestamp) - 12, "%5d (PID) - %s", cur -> pid, timestamp);
+						mvwprintw(status -> win, 0, win_width - strlen(timestamp) - 12, "%5d (PID) - %s", cur -> pid, timestamp);
 				}
 			}
 			else if (fsize == -1)
 			{
 				if (cur -> wt == WT_STDIN || cur -> wt == WT_SOCKET)
-					mvwprintw(status -> win, 0, dx - strlen(timestamp), "%s", timestamp);
+					mvwprintw(status -> win, 0, win_width - strlen(timestamp), "%s", timestamp);
 				else
 				{
-					mvwprintw(status -> win, 0, dx - strlen(timestamp) - 6, "??? - %s", timestamp);
+					mvwprintw(status -> win, 0, win_width - strlen(timestamp) - 6, "??? - %s", timestamp);
 					total_info_len = statusline_len + 6;
 				}
 			}
@@ -1446,7 +1443,7 @@ void update_statusline(NEWWIN *status, int win_nr, proginfo *cur)
 				{
 					char *filesize = amount_to_str(fsize);
 					cur_len = 3 + strlen(filesize);
-					mvwprintw(status -> win, 0, dx - (strlen(timestamp) + cur_len), "%s - %s", filesize, timestamp);
+					mvwprintw(status -> win, 0, win_width - (strlen(timestamp) + cur_len), "%s - %s", filesize, timestamp);
 					myfree(filesize);
 				}
 				else
@@ -1457,9 +1454,9 @@ void update_statusline(NEWWIN *status, int win_nr, proginfo *cur)
 					/* this trick is because on MacOS X 'off_t' is specified as a 64 bit integer */
 #endif
 					if (sizeof(off64_t) == 8)
-						mvwprintw(status -> win, 0, dx - (strlen(timestamp) + cur_len), "%10lld - %s", fsize, timestamp);
+						mvwprintw(status -> win, 0, win_width - (strlen(timestamp) + cur_len), "%10lld - %s", fsize, timestamp);
 					else
-						mvwprintw(status -> win, 0, dx - (strlen(timestamp) + cur_len), "%10ld - %s", fsize, timestamp);
+						mvwprintw(status -> win, 0, win_width - (strlen(timestamp) + cur_len), "%10ld - %s", fsize, timestamp);
 				}
 
 				total_info_len = statusline_len + cur_len;
@@ -1469,11 +1466,11 @@ void update_statusline(NEWWIN *status, int win_nr, proginfo *cur)
 		if (show_f1)
 		{
 			if (use_colors) color_on(status, find_colorpair(COLOR_YELLOW, -1, 0));
-			if (dx >= (total_info_len + 32))
+			if (win_width >= (total_info_len + 32))
 				mvwprintw(status -> win, 0, help_str_offset, " *Press F1/<CTRL>+<h> for help* ");
-			else if (dx >= (total_info_len + 21))
+			else if (win_width >= (total_info_len + 21))
 				mvwprintw(status -> win, 0, help_str_offset, " F1/<CTRL>+<h>: help ");
-			else if (dx >= (total_info_len + 13))
+			else if (win_width >= (total_info_len + 13))
 				mvwprintw(status -> win, 0, help_str_offset, " F1/^h: help ");
 			if (use_colors) color_off(status, find_colorpair(COLOR_YELLOW, -1, 0));
 		}
@@ -2014,6 +2011,7 @@ void do_restart_window(proginfo *cur)
 	myclose(cur -> wfd);
 
 	/* re-start tail */
+	cur -> restart.first = 1;
 	if (start_proc(cur, 1) == -1)
 		error_exit(FALSE, FALSE, "Failed to start process %s.\n", cur -> filename);
 }
@@ -3367,6 +3365,40 @@ int recv_from_fd(int fd, char **buffer, int new_data_offset, int read_size)
 	return nbytes;
 }
 
+void get_markerline_sleep(double *const s, char *have_any)
+{
+	int loop = 0;
+	dtime_t now = get_ts();
+
+	*have_any = 0;
+
+	for(loop=0; loop<nfd; loop++)
+	{
+		proginfo *cur = &pi[loop];
+
+		do
+		{
+			if (cur -> mark_interval)
+			{
+				double t_until_next = cur -> statistics.lastevent + cur -> mark_interval - now;
+
+				*have_any = 1;
+
+				if (t_until_next <= 0)
+				{
+					*s = 0;
+					return;
+				}
+
+				*s = min(*s, t_until_next);
+			}
+
+			cur = cur -> next;
+		}
+		while(cur);
+	}
+}
+
 int wait_for_keypress(int what_help, double max_wait, NEWWIN *popup, char cursor_shift)
 {
 	int c = -1;
@@ -3383,6 +3415,7 @@ int wait_for_keypress(int what_help, double max_wait, NEWWIN *popup, char cursor
 		char prev_mail_status = mail;
 		static double lastupdate = 0;
 		double sleep = 32767.0;
+		char any_markers = 0;
 
 		/* need to check any paths? */
 		if (check_paths())
@@ -3392,6 +3425,8 @@ int wait_for_keypress(int what_help, double max_wait, NEWWIN *popup, char cursor
 		sleep = min(sleep, heartbeat_interval > 0 ? max((heartbeat_t + heartbeat_interval) - now, 0) : 32767);
 		sleep = min(sleep, max_wait > 0.0         ? max((max_wait_start + max_wait)        - now, 0) : 32767);
 
+		get_markerline_sleep(&sleep, &any_markers);
+
 		for(loop=0; loop<n_cdg; loop++)
 			sleep = min(sleep, max((cdg[loop].last_check + cdg[loop].check_interval) - now, 0));
 
@@ -3494,6 +3529,36 @@ int wait_for_keypress(int what_help, double max_wait, NEWWIN *popup, char cursor
 			sigusr1_restart_tails();
 		}
 
+		if (any_markers)
+		{
+			for(loop=0; loop<nfd; loop++)
+			{
+				proginfo *cur = &pi[loop];
+
+				if (cur -> paused || cur -> closed)
+					continue;
+
+				do
+				{
+					if (cur -> mark_interval && now - cur -> statistics.lastevent >= cur -> mark_interval)
+					{
+						if (cur -> statistics.lastevent)
+						{
+							add_markerline(loop, cur, MARKER_IDLE, NULL);
+
+							if (get_do_refresh() == 0)
+								set_do_refresh(1);
+						}
+
+						cur -> statistics.lastevent = now;
+					}
+
+					cur = cur -> next;
+				}
+				while(cur);
+			}
+		}
+
 		if (rc > 0)
 		{
 			/* any key pressed? */
@@ -3600,19 +3665,6 @@ int wait_for_keypress(int what_help, double max_wait, NEWWIN *popup, char cursor
 						}
 					}
 
-					if (cur -> mark_interval)
-					{
-						if (now - cur -> statistics.lastevent >= cur -> mark_interval)
-						{
-							add_markerline(loop, cur, MARKER_IDLE, NULL);
-
-							if (get_do_refresh() == 0)
-								set_do_refresh(1);
-
-							cur -> statistics.lastevent = now;
-						}
-					}
-
 					cur = cur -> next;
 				}
 				while(cur);
