@@ -19,6 +19,7 @@
 #include "ui.h"
 #include "misc.h"
 #include "globals.h"
+#include "xclip.h"
 
 int scrollback_search_to_new_window(buffer *pbuf, char *org_title, char *find_str, mybool_t case_insensitive);
 
@@ -222,7 +223,8 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 	char show_winnr = default_sb_showwinnr;
 	mybool_t case_insensitive = re_case_insensitive;
 	buffer cur_lb;
-	int loop;
+	int loop = 0;
+	char fullscreen = 0;
 
 	memset(&cur_lb, 0x00, sizeof(cur_lb));
 
@@ -253,7 +255,6 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 	}
 
 	mywin1 = create_popup(max_y - 4, max_x - 4);
-
 	mywin2 = create_popup(nlines, ncols);
 	scrollok(mywin2 -> win, FALSE); /* supposed to always return OK, according to the manpage */
 
@@ -266,13 +267,16 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			int index = 0;
 			int lines_used = 0;
 
-			ui_inverse_on(mywin1);
-			mvwprintw(mywin1 -> win, nlines + 1, 1, "%s - %d buffered lines", shorten_filename(header, max(24, ncols - 24)), cur_lb.curpos);
-			ui_inverse_off(mywin1);
+			if (mywin1)
+			{
+				ui_inverse_on(mywin1);
+				mvwprintw(mywin1 -> win, nlines + 1, 1, "%s - %d buffered lines", shorten_filename(header, max(24, ncols - 24)), cur_lb.curpos);
+				ui_inverse_off(mywin1);
 
-			if (!no_linewrap) ui_inverse_on(mywin1);
-			mvwprintw(mywin1 -> win, nlines + 1, ncols - 8, "LINEWRAP");
-			if (!no_linewrap) ui_inverse_off(mywin1);
+				if (!no_linewrap) ui_inverse_on(mywin1);
+				mvwprintw(mywin1 -> win, nlines + 1, ncols - 8, "LINEWRAP");
+				if (!no_linewrap) ui_inverse_off(mywin1);
+			}
 
 			werase(mywin2 -> win);
 
@@ -332,7 +336,7 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 		c = wait_for_keypress(HELP_SCROLLBACK_HELP, 0, NULL, 1);
 		uc = toupper(c);
 
-		if (c == 'q' || uc == 'X' || c == abort_key || c == KEY_CLOSE || c == KEY_EXIT)
+		if (c == 'q' || c == abort_key || c == KEY_CLOSE || c == KEY_EXIT)
 		{
 			break;
 		}
@@ -346,19 +350,45 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			show_winnr = 1 - show_winnr;
 			redraw = 2;
 		}
+		else if (c == 'x')
+		{
+			send_to_clipboard(pbuf);
+		}
 		else if (c == 'Y')
 		{
 			no_linewrap = !no_linewrap;
 			redraw = 2;
 			line_offset = 0;
 		}
+		else if (c == KEY_F(9) || c == 23)	/* ^w */
+		{
+			if (fullscreen)
+			{
+				delete_popup(mywin2);
+
+				mywin1 = create_popup(max_y - 4, max_x - 4);
+				mywin2 = create_popup(nlines, ncols);
+				scrollok(mywin2 -> win, FALSE); /* supposed to always return OK, according to the manpage */
+			}
+			else
+			{
+				delete_popup(mywin1);
+				delete_popup(mywin2);
+
+				mywin1 = NULL;
+				mywin2 = create_popup(max_y, max_x);
+				scrollok(mywin2 -> win, FALSE); /* supposed to always return OK, according to the manpage */
+			}
+
+			fullscreen = ! fullscreen;
+
+			redraw = 2;
+		}
 		else if (c == 't')
 		{
 			statistics_menu();
 		}
-		else if ((c == KEY_LEFT ||
-					c == KEY_BACKSPACE)
-				&& no_linewrap)
+		else if ((c == KEY_LEFT || c == KEY_BACKSPACE) && no_linewrap)
 		{
 			if (line_offset > 0)
 				line_offset--;
@@ -692,7 +722,8 @@ int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 	}
 
 	delete_popup(mywin2);
-	delete_popup(mywin1);
+	if (mywin1)
+		delete_popup(mywin1);
 
 	myfree(find);
 
