@@ -45,7 +45,7 @@ int find_path_max(void)
 	int path_max = pathconf("/", _PC_PATH_MAX);
 	if (path_max <= 0)
 	{
-		if (errno) error_exit("pathconf() failed\n");
+		if (errno) error_exit(TRUE, FALSE, "pathconf() failed\n");
 
 		path_max = 4096;
 	}
@@ -76,7 +76,7 @@ ssize_t WRITE(int fd, char *whereto, size_t len, char *for_whom)
 		if (rc == -1)
 		{
 			if (errno != EINTR && errno != EAGAIN)
-				error_exit("Problem writing to file descriptor while processing %s.\n", for_whom);
+				error_exit(TRUE, FALSE, "Problem writing to file descriptor while processing %s.\n", for_whom);
 		}
 		else if (rc == 0)
 		{
@@ -108,7 +108,7 @@ void get_load_values(double *v1, double *v2, double *v3)
 		 * calls start to fail, something must be
 		 * really wrong
 		 */
-		error_exit("sysinfo() failed\n");
+		error_exit(TRUE, FALSE, "sysinfo() failed\n");
 	}
 
 	*v1 = (double)si.loads[0] / scale;
@@ -119,7 +119,7 @@ void get_load_values(double *v1, double *v2, double *v3)
 	if (getloadavg(loadavg, 3) == -1)
 	{
 		/* see comment on sysinfo() */
-		error_exit("getloadavg() failed\n");
+		error_exit(TRUE, FALSE, "getloadavg() failed\n");
 	}
 	*v1 = loadavg[0];
 	*v2 = loadavg[1];
@@ -187,7 +187,7 @@ void stop_process(pid_t pid)
 	if (mykillpg(pid, SIGTERM) == -1)
 	{
 		if (errno != ESRCH)
-			error_exit("Problem stopping child process with PID %d (SIGTERM).\n", pid);
+			error_exit(TRUE, FALSE, "Problem stopping child process with PID %d (SIGTERM).\n", pid);
 	}
 
 	usleep(1000);
@@ -202,11 +202,11 @@ void stop_process(pid_t pid)
 		if (mykillpg(pid, SIGKILL) == -1)
 		{
 			if (errno != ESRCH)
-				error_exit("Problem stopping child process with PID %d (SIGKILL).\n", pid);
+				error_exit(TRUE, FALSE, "Problem stopping child process with PID %d (SIGKILL).\n", pid);
 		}
 	}
 	else if (errno != ESRCH)
-		error_exit("Problem stopping child process with PID %d (SIGTERM).\n", pid);
+		error_exit(TRUE, FALSE, "Problem stopping child process with PID %d (SIGTERM).\n", pid);
 
 	/* wait for the last remainder of the died process to go away,
 	 * otherwhise we'll find zombies on our way
@@ -214,7 +214,7 @@ void stop_process(pid_t pid)
 	if (waitpid(pid, NULL, WNOHANG | WUNTRACED) == -1)
 	{
 		if (errno != ECHILD)
-			error_exit("waitpid() failed\n");
+			error_exit(TRUE, FALSE, "waitpid() failed\n");
 	}
 }
 
@@ -257,12 +257,13 @@ int file_info(char *filename, off64_t *file_size, time_field_t tft, time_t *ts,
 	if (stat64(filename, &buf) == -1)
 	{
 		if (errno != ENOENT)
-			error_exit("Error while obtaining details of file %s.\n", filename);
+			error_exit(TRUE, FALSE, "Error while obtaining details of file %s.\n", filename);
 
 		return -1;
 	}
 
-	if (file_size) *file_size = buf.st_size;
+	if (file_size)
+		*file_size = buf.st_size;
 
 	if (ts)
 	{
@@ -275,10 +276,12 @@ int file_info(char *filename, off64_t *file_size, time_field_t tft, time_t *ts,
 		else
 		{
 			assert(tft == 0);
+			*ts = 0;
 		}
 	}
 
-	if (mode) *mode = buf.st_mode;
+	if (mode)
+		*mode = buf.st_mode;
 
 	return 0;
 }
@@ -289,7 +292,7 @@ int file_exist(char *filename)
 	int rc = stat(filename, &buf);
 
 	if (rc == -1 && errno != ENOENT)
-		error_exit("stat() on file %s failed.\n", filename);
+		error_exit(TRUE, FALSE, "stat() on file %s failed.\n", filename);
 
 	return rc;
 }
@@ -337,21 +340,32 @@ char * amount_to_str(long long int amount)
 
 struct passwd *getuserinfo(void)
 {
-	struct passwd *pp = getpwuid(geteuid());
-	if (!pp)
-		error_exit("Failed to get passwd-structure for effective user id %d.\n", geteuid());
-
-	return pp;
+	return getpwuid(geteuid());
 }
 
 char * getusername(void)
 {
-	static char username[128] = { 0 };
+	static char username[128] = "???";
 
-	if (!username[0])
+	if (username[0] == '?')
 	{
-		strncpy(username, getuserinfo() -> pw_name, sizeof(username));
-		username[sizeof(username) - 1] = 0x00;
+		char *logname = getenv("LOGNAME");
+		if (logname)
+		{
+			strncpy(username, logname, 128);
+			username[127] = 0x00;
+		}
+	}
+
+	if (username[0] == '?')
+	{
+		struct passwd *pw = getuserinfo();
+
+		if (pw)
+		{
+			strncpy(username, getuserinfo() -> pw_name, sizeof(username));
+			username[sizeof(username) - 1] = 0x00;
+		}
 	}
 
 	return username;
@@ -423,7 +437,7 @@ double get_ts(void)
 	struct timezone tz;
 
 	if (gettimeofday(&ts, &tz) == -1)
-		error_exit("gettimeofday() failed");
+		error_exit(TRUE, FALSE, "gettimeofday() failed");
 
 	return (((double)ts.tv_sec) + ((double)ts.tv_usec)/1000000.0);
 }
@@ -480,7 +494,7 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 			if (stat(cur_dir, &finfo) == -1)
 			{
 				if (errno != ENOENT)	/* file did not disappear? then something is very wrong */
-					error_exit("Error while invoking stat() %s.\n", cur_dir);
+					error_exit(TRUE, FALSE, "Error while invoking stat() %s.\n", cur_dir);
 			}
 			(*isdir)[nfound] = S_ISDIR(finfo.st_mode)?1:0;
 
@@ -489,7 +503,7 @@ int match_files(char *search_for, char **path, char ***found, char **isdir)
 	}
 
 	if (closedir(dir) == -1)
-		error_exit("closedir() failed\n");
+		error_exit(TRUE, FALSE, "closedir() failed\n");
 
 	/*	qsort( (void *)list, (size_t)nfound, sizeof(char *), compare_filenames); */
 	for(s1=0; s1<(nfound - 1); s1++)
@@ -521,12 +535,12 @@ void setup_for_childproc(int fd, char close_fd_0, char *term)
 	int term_len = strlen(term) + 5;
 	char *dummy = (char *)mymalloc(term_len + 1);
 
-	if (close_fd_0) if (-1 == myclose(0)) error_exit("close() failed\n");
-	if (-1 == myclose(1)) error_exit("close() failed\n");
-	if (-1 == myclose(2)) error_exit("close() failed\n");
-	if (close_fd_0) if (-1 == mydup(fd)) error_exit("dup() failed\n");
-	if (-1 == mydup(fd)) error_exit("dup() failed\n");
-	if (-1 == mydup(fd)) error_exit("dup() failed\n");
+	if (close_fd_0) if (-1 == myclose(0)) error_exit(TRUE, FALSE, "close() failed\n");
+	if (-1 == myclose(1)) error_exit(TRUE, FALSE, "close() failed\n");
+	if (-1 == myclose(2)) error_exit(TRUE, FALSE, "close() failed\n");
+	if (close_fd_0) if (-1 == mydup(fd)) error_exit(TRUE, FALSE, "dup() failed\n");
+	if (-1 == mydup(fd)) error_exit(TRUE, FALSE, "dup() failed\n");
+	if (-1 == mydup(fd)) error_exit(TRUE, FALSE, "dup() failed\n");
 
 	/*
 	 * not doing this: it also clears the 'PATH'
@@ -554,7 +568,7 @@ int open_null(void)
 {
 	int fd = myopen("/dev/null", O_RDWR);
 	if (fd == -1)
-		error_exit("Failed to open file /dev/null.\n");
+		error_exit(TRUE, FALSE, "Failed to open file /dev/null.\n");
 
 	return fd;
 }
@@ -666,7 +680,7 @@ int mydup(int old_fd)
 			if (errno == EINTR)
 				continue;
 
-			error_exit("dup() failed\n");
+			error_exit(TRUE, FALSE, "dup() failed\n");
 		}
 
 		break;
@@ -696,7 +710,7 @@ void double_ts_to_str(dtime_t ts, char *format_str, char *dest, int dest_size)
 {
 	time_t now = (time_t)ts;
 	struct tm *ptm = localtime(&now);
-	if (!ptm) error_exit("localtime() failed\n");
+	if (!ptm) error_exit(TRUE, FALSE, "localtime() failed\n");
 
 	assert(ts > 0);
 	assert(dest_size > 0);
@@ -795,7 +809,7 @@ void compile_re(regex_t *whereto, char *what)
 	/* compile & store regular expression */
 	int rc = regcomp(whereto, what, REG_EXTENDED);
 	if (rc != 0)
-		error_exit("Failed to compile regular expression '%s'.\nReason: %s\n", what, convert_regexp_error(rc, whereto));
+		error_exit(FALSE, FALSE, "Failed to compile regular expression '%s'.\nReason: %s\n", what, convert_regexp_error(rc, whereto));
 }
 
 void grow_mem_if_needed(char **what, int *cur_len, int requested_len)
@@ -829,7 +843,7 @@ int READ(int fd, char *where_to, int max_len, char *for_whom)
 			return rc;
 
 		if (errno != EINTR && errno != EAGAIN)
-			error_exit("Error writing to fd for %s.\n", for_whom);
+			error_exit(TRUE, FALSE, "Error writing to fd for %s.\n", for_whom);
 	}
 }
 
@@ -840,7 +854,7 @@ int get_value_arg(char *par, char *string, valcheck_t check)
 	int multiplier = 1;
 
 	if (!string)
-		error_exit("%s needs a parameter.\n", par);
+		error_exit(FALSE, FALSE, "%s needs a parameter.\n", par);
 
 	len = strlen(string);
 
@@ -862,31 +876,31 @@ int get_value_arg(char *par, char *string, valcheck_t check)
 				break;
 			}
 			else
-				error_exit("%s needs a value as parameter.\n", par);
+				error_exit(FALSE, FALSE, "%s needs a value as parameter.\n", par);
 		}
 	}
 
 	result = strtol(string, NULL, 10);
 
 	if (result > INT_MAX || result == LONG_MIN || result == LONG_MAX)
-		error_exit("Value %s for parameter %s is too large.\n", string, par);
+		error_exit(FALSE, FALSE, "Value %s for parameter %s is too large.\n", string, par);
 
 	result *= multiplier;
 
 	if (check == VAL_ZERO_POSITIVE)
 	{
 		if (result < 0)
-			error_exit("Value for %s must be >= 0.\n", par);
+			error_exit(FALSE, FALSE, "Value for %s must be >= 0.\n", par);
 	}
 	else if (check == VAL_POSITIVE_NOT_1)
 	{
 		if (result < 0 || result == 1)
-			error_exit("Value for %s must be >= 0 and must not be 1.\n", par);
+			error_exit(FALSE, FALSE, "Value for %s must be >= 0 and must not be 1.\n", par);
 	}
 	else if (check == VAL_POSITIVE)
 	{
 		if (result <= 0)
-			error_exit("Value for %s must be > 0.\n", par);
+			error_exit(FALSE, FALSE, "Value for %s must be > 0.\n", par);
 	}
 	else
 	{
@@ -984,7 +998,7 @@ char *myrealpath(char *in)
 	}
 
 	if (!home)
-		error_exit("Cannot expand path %s:\nhome directory not found.\n", in);
+		error_exit(FALSE, FALSE, "Cannot expand path %s:\nhome directory not found.\n", in);
 
 	home_len = strlen(home);
 	pout = (char *)mymalloc(home_len + strlen(in) + 1);
@@ -995,3 +1009,21 @@ char *myrealpath(char *in)
 
 	return pout;
 }
+
+void str_add(char **to, const char *what, ...)
+{
+	int len_to = *to ? strlen(*to) : 0;
+	char buffer[4096] = { 0 };
+	int len_what = 0;
+
+        va_list ap;
+
+	/* FIXME: should vasprintf here: does other unixes have it? */
+        va_start(ap, what);
+        len_what = vsnprintf(buffer, sizeof buffer, what, ap);
+        va_end(ap);
+
+	*to = (char *)realloc(*to, len_to + len_what + 1);
+
+	memcpy(&(*to)[len_to], buffer, len_what + 1);
+}
